# Concept: Move semantics

Move semantics, introduced in C++11, are a key feature that optimizes the performance of C++ programs by allowing the resources of temporary objects to be "moved" rather than copied. This is especially useful for large objects, where copying can be expensive in terms of performance. Move semantics work with a special type of constructor and assignment operator known as the move constructor and move assignment operator.

## Key Concepts of Move Semantics

### Rvalue and Lvalue:
- Lvalue: An object that occupies some identifiable location in memory (e.g., a variable, array element, etc.).
- Rvalue: A temporary object or a literal that does not have a persistent memory location (e.g., 3, `x + y`, `std::string("hello")`).

### Move Constructor: 

A constructor that "moves" resources from a temporary object (an rvalue) to a new object, leaving the original object in a valid but unspecified state.

### Move Assignment Operator: 

Similar to the move constructor, but used for assigning an rvalue to an already existing object.

## Benefits of Move Semantics

- Efficiency: Move semantics avoid unnecessary deep copies of objects, which can be particularly expensive for objects managing large resources like dynamic memory, file handles, or network connections.
- Improved Performance: Especially beneficial in scenarios involving temporary objects, such as when returning large objects from functions or during container operations (e.g., `std::vector` reallocation).

## Example 1: Move Constructor

Let's start with a simple example where move semantics significantly improve performance.

```cpp
#include <iostream>
#include <vector>

class MyClass {
public:
    int* data;
    std::size_t size;

    // Constructor
    MyClass(std::size_t s) : size(s) {
        data = new int[s];
        std::cout << "Constructed with size: " << size << std::endl;
    }

    // Destructor
    ~MyClass() {
        delete[] data;
        std::cout << "Destroyed" << std::endl;
    }

    // Copy Constructor
    MyClass(const MyClass& other) : size(other.size) {
        data = new int[size];
        std::copy(other.data, other.data + size, data);
        std::cout << "Copy Constructed" << std::endl;
    }

    // Move Constructor
    MyClass(MyClass&& other) noexcept : data(nullptr), size(0) {
        data = other.data;
        size = other.size;
        other.data = nullptr;
        other.size = 0;
        std::cout << "Move Constructed" << std::endl;
    }

    // Copy Assignment Operator
    MyClass& operator=(const MyClass& other) {
        if (this == &other)
            return *this;

        delete[] data;
        size = other.size;
        data = new int[size];
        std::copy(other.data, other.data + size, data);
        std::cout << "Copy Assigned" << std::endl;
        return *this;
    }

    // Move Assignment Operator
    MyClass& operator=(MyClass&& other) noexcept {
        if (this == &other)
            return *this;

        delete[] data;

        data = other.data;
        size = other.size;
        other.data = nullptr;
        other.size = 0;
        std::cout << "Move Assigned" << std::endl;
        return *this;
    }
};

int main() {
    MyClass obj1(10);
    MyClass obj2 = std::move(obj1);  // Move construction

    MyClass obj3(20);
    obj3 = std::move(obj2);  // Move assignment

    return 0;
}
```

### Explanation:

- Copy Constructor: Performs a deep copy of the data array from one object to another. This is costly for large arrays.
- Move Constructor: Transfers ownership of the data array from a temporary (rvalue) object to a new object. The original object is left in a safe state, typically with nullptr and size 0.
- Move Assignment Operator: Transfers ownership of the data array from a temporary object to an existing object, freeing any existing resources in the target object.
- std::move: Explicitly casts an object to an rvalue reference, enabling the move constructor or move assignment operator to be invoked.

## When to Use Move Semantics

- Resource Management: When your class manages resources like dynamic memory, file handles, or any other non-trivial resource.
- Performance Critical Code: In situations where performance is critical and deep copies would be costly.
- Temporary Objects: When functions return large objects by value, move semantics can optimize the transfer of data, avoiding unnecessary copies.

## Example 2: Move Semantics in Standard Library Containers

Standard containers like `std::vector`, `std::string`, and others leverage move semantics to improve performance during operations like resizing or reallocation.

```cpp
#include <iostream>
#include <vector>

int main() {
    std::vector<MyClass> vec;
    vec.reserve(2);

    MyClass obj1(10);
    vec.push_back(std::move(obj1));  // Uses move constructor

    MyClass obj2(20);
    vec.push_back(std::move(obj2));  // Uses move constructor

    return 0;
}
```

### Explanation:

- `std::move(obj1)`: Casts `obj1` to an rvalue, so `vec.push_back()` uses the move constructor instead of the copy constructor.
- When a `std::vecto`r needs to reallocate its storage, it can move objects to the new storage, improving performance.

## Common Pitfalls

- Dangling References: After moving from an object, it's typically in a valid but unspecified state. Using it without proper checks (e.g., accessing data after it's been moved) can lead to undefined behavior.
- Self-assignment: Always check for self-assignment in move assignment operators to prevent unwanted behavior.
- Exceptions: The noexcept specifier is important in move constructors and move assignment operators. If these operations can throw exceptions, it may prevent some optimizations (e.g., in standard containers).

## Summary

- Move Semantics: Optimize the handling of temporary objects and prevent unnecessary deep copies by transferring ownership of resources from one object to another.
- Move Constructor: A special constructor that transfers ownership of resources from an rvalue to a new object.
- Move Assignment Operator: Similar to the move constructor but used to transfer resources to an existing object.
- Performance Benefits: Move semantics are especially beneficial in performance-critical applications where copying large objects is expensive.

Move semantics are a cornerstone of modern C++ programming, allowing for more efficient code by eliminating unnecessary copies and enabling resource ownership transfer.

